@@ -1,4 +1,4 @@
-# jonashaslbeck@protonmail; June 2nd, 2023
+# jonashaslbeck@protonmail; May 17th, 2024
 
 # ------------------------------------------------------------
 # -------- Function for Permutation Test ---------------------
@@ -19,13 +19,17 @@ mlVAR_GC <- function(data, # data including both groups
                      beepvar = NULL,
                      groups, # indicates which case belongs to which group
                      test = "permutation", # can also be "parametric"
+                     partest = "Welch",
                      paired = FALSE,
                      estimator, # same as in ml
                      contemporaneous, # same as in ml
                      temporal, # same as in ml
+                     scale, # is data scaled?
+                     # trueMeans, # allow specifying true means for developmental purposes
                      nCores = 1,
                      nP = 500, # number of samples in permutation test
                      saveModels = FALSE, # if TRUE, all models are saved; defaults to FALSE to save memory
+                     saveEmpModels = FALSE,
                      verbose = FALSE, # if TRUE, verbose mode is activated in foreach
                      pbar = TRUE # if TRUE, a progress bar is being shown
 ) {
@@ -52,6 +56,8 @@ mlVAR_GC <- function(data, # data including both groups
   # (5) No paired parametric test
   if(test == "parametric" & paired == TRUE) stop("No parametric test available for paired samples. Use the paired permutation test instead.")
 
+  # (6) Specification of "beepvar" and "dayvar"
+  if(!is.null(beepvar) & is.null(dayvar)) stop("If you use only one time vector, use 'dayvar' and not 'beepvar')")
 
   data1 <- data[v_groups==1, ]
   data2 <- data[v_groups==2, ]
@@ -63,12 +69,20 @@ mlVAR_GC <- function(data, # data including both groups
   v_u_ids <- unique(v_ids)
   u_ids1 <- unique(ids1)
   u_ids2 <- unique(ids2)
-  n_subj <- length(u_ids1) + length(u_ids2)
+  n_subj_G1 <- length(u_ids1)
+  n_subj_G2 <- length(u_ids2)
+  n_subj <- n_subj_G1 + n_subj_G2
 
-  # (6) Are IDs unique across datasets? [required for indepdendent samples]
+  # (6) Independent samples test: Are IDs unique across datasets? [required for indepdendent samples]
   if(paired == FALSE) {
-  v_intersec <- intersect(u_ids1, u_ids2)
-  if(length(v_intersec) > 0) stop("IDs need to be unique across two datasets.")
+    v_intersec <- intersect(u_ids1, u_ids2)
+    if(length(v_intersec) > 0) stop("IDs need to be unique across two datasets.")
+  }
+
+  # (7) Paired samples test: do we have measurements in both "groups" form each person?
+  if(paired == TRUE) {
+    ind_same_IDs <- all(sort(u_ids1) == sort(u_ids2))
+    if(!ind_same_IDs) stop("Data for each ID is required in both 'groups'.")
   }
 
   # ------ Collect passed down arguments -----
@@ -76,6 +90,8 @@ mlVAR_GC <- function(data, # data including both groups
   if(missing(contemporaneous)) contemporaneous <- "orthogonal"
   if(missing(temporal)) temporal <- "orthogonal"
   if(missing(nCores)) nCores <- 1
+  if(missing(scale)) scale <- TRUE
+  # if(missing(trueMeans)) trueMeans <- NULL
 
   # Copy call
   Call <- list("vars" = vars,
@@ -87,6 +103,7 @@ mlVAR_GC <- function(data, # data including both groups
                "estimator" = estimator,
                "contemporaneous" = contemporaneous,
                "temporal" = temporal,
+               "scale" = scale,
                "nCores" = nCores,
                "nP" = nP,
                "saveModels" = saveModels,
@@ -124,19 +141,29 @@ mlVAR_GC <- function(data, # data including both groups
       registerDoParallel(cl)
     }
 
+    # Define a function that suppresses package startup messages
+    quiet_library <- function(pkg) {
+      suppressPackageStartupMessages(library(pkg, character.only = TRUE))
+    }
+
     # Call foreach:
     out_P <- foreach(b = 1:nP,
-                     .packages = c("mlVAR", "mnet"),
+                     .packages = c("mlVAR"),
                      .export = c("m_data_cmb", "vars", "idvar", "estimator",
                                  "contemporaneous", "temporal", "totalN", "v_Ns",
-                                 "v_ids", "pb", "pbar", "dayvar", "beepvar", "paired"),
+                                 "v_ids", "pb", "pbar", "dayvar", "beepvar", "paired",
+                                 "quiet_library"),
                      .verbose = verbose) %dopar% {
 
+                       # Suppress starting message of mnet
+                       quiet_library("mnet")
 
                        # --- Make permutation ---
 
                        # For independent samples
                        if(paired == FALSE) {
+                         # Here we randomly distribute persons across the group, keeping with the original group sizes
+
                          # This is done in a way that keeps the size in each group exactly the same as in the real groups
                          v_ids_rnd <- v_u_ids[sample(1:totalN, size=totalN, replace=FALSE)]
                          v_ids_1 <- v_ids_rnd[1:v_Ns[1]]
@@ -150,31 +177,24 @@ mlVAR_GC <- function(data, # data including both groups
 
                        # For dependent samples
                        if(paired == TRUE) {
+                         # here we go through each participant, and switch the group label (or not), determined by a coin flip
 
-                         ids1 <- sapply(data1[, idvar], as.character)
-                         ids2 <- sapply(data2[, idvar], as.character)
-                         v_ids <- c(ids1, ids2)
+                         unique_IDs <- unique(data[, idvar])
+                         n_unique_IDs <- length(unique_IDs)
 
-                         ids1_uq <- unique(ids1)
-                         ids2_uq <- unique(ids2)
-                         m_ids_uq <- cbind(ids1_uq, ids2_uq)
-                         n_uq <- nrow(m_ids_uq)
-                         m_ids_uq_perm <- matrix(NA, n_uq, 2)
-                         for(i in 1:n_uq) {
-                           draw_i <- sample(1:2, size=1)
-                           if(draw_i==1) m_ids_uq_perm[i, ] <- m_ids_uq[i, ] else  m_ids_uq_perm[i, ] <- m_ids_uq[i, 2:1]
-                         }
-                         v_ids_1 <- m_ids_uq_perm[, 1]
-                         v_ids_2 <- m_ids_uq_perm[, 2]
+                         # Create copies of dataset
+                         data_perm <- data
 
-                         # Split data based on permutations
-                         data_h0_1 <- m_data_cmb[v_ids %in% v_ids_1, ]
-                         data_h0_2 <- m_data_cmb[v_ids %in% v_ids_2, ]
-                         l_data_h0 <- list(data_h0_1, data_h0_2)
-                       } # end if
+                         for(i in 1:n_unique_IDs) {
+                           flip <- sample(c(FALSE, TRUE), size=1, prob = c(0.5, 0.5))
+                           if(flip) data_perm[data_perm[, idvar] == unique_IDs[i], groups] <- abs(data_perm[data_perm[, idvar] == unique_IDs[i], groups] -3)
+                         } # end: permuting
 
-                       # browser()
+                         # Collect two permuted datasets in list
+                         l_data_h0 <- list(data_perm[data_perm[, groups]==1, ],
+                                           data_perm[data_perm[, groups]==2, ])
 
+                       } # end if: paired?
 
 
                        # --- Fit mlVAR models ---
@@ -186,28 +206,50 @@ mlVAR_GC <- function(data, # data including both groups
                        for(j in 1:2) {
 
                          # TODO: make this variable specification of dayvar/beepvar less hacky
-                         if(is.null(dayvar)) {
+
+                         # Neither daybvar/beepvar are specified
+                         if(is.null(dayvar) & is.null(beepvar)) {
                            l_pair_b[[j]] <- mlVAR(data = l_data_h0[[j]],
                                                   vars = vars,
                                                   idvar = idvar,
                                                   estimator = estimator,
                                                   contemporaneous = contemporaneous,
                                                   temporal = temporal,
+                                                  scale = scale,
                                                   nCores = 1, # we use parallelization across resamples (not nodes in nodewise estimation here)
                                                   verbose = FALSE,
-                                                  lags = 1) # TODO: later allow also higher order lags (see also below)
-                         } else {
+                                                  lags = 1)
+                           # Only dayvar is specified
+                         } else if (!is.null(dayvar) & is.null(beepvar)) {
                            l_pair_b[[j]] <- mlVAR(data = l_data_h0[[j]],
                                                   vars = vars,
                                                   idvar = idvar,
                                                   estimator = estimator,
                                                   contemporaneous = contemporaneous,
                                                   temporal = temporal,
+                                                  scale = scale,
+                                                  nCores = 1,
+                                                  dayvar = dayvar, # now also provided
+                                                  # beepvar = beepvar,
+                                                  verbose = FALSE,
+                                                  lags = 1)
+                           # Both dayvar and beepvar are specified
+                         } else {
+
+                           l_pair_b[[j]] <- mlVAR(data = l_data_h0[[j]],
+                                                  vars = vars,
+                                                  idvar = idvar,
+                                                  estimator = estimator,
+                                                  contemporaneous = contemporaneous,
+                                                  temporal = temporal,
+                                                  scale = scale,
                                                   nCores = 1,
                                                   dayvar = dayvar, # now also provided
                                                   beepvar = beepvar,
                                                   verbose = FALSE,
-                                                  lags = 1) # TODO: later allow also higher order lags (see also below)
+                                                  lags = 1)
+
+
                          } # end if: dayvar specified
 
                          if(saveModels) l_models[[j]] <- l_pair_b[[j]]
@@ -276,31 +318,111 @@ mlVAR_GC <- function(data, # data including both groups
 
   for(j in 1:2) {
 
+    # browser()
+
     # TODO: make this variable specification of dayvar/beepvar less hacky
     if(is.null(dayvar)) {
+      # if(is.null(trueMeans)) {
+
+      # neither dayvar/beepvar are specified
       l_out_emp[[j]] <-  mlVAR(data = l_data[[j]],
                                vars = vars,
                                idvar = idvar,
                                estimator = estimator,
                                contemporaneous = contemporaneous,
                                temporal = temporal,
+                               scale = scale,
                                nCores = 1,
                                verbose = FALSE,
                                lags = 1)
-    } else {
+
+      # Only dayvar is specified
+    } else if (!is.null(dayvar) & is.null(beepvar)) {
+
+      # Fit mlVAR
       l_out_emp[[j]] <-  mlVAR(data = l_data[[j]],
                                vars = vars,
                                idvar = idvar,
                                estimator = estimator,
                                contemporaneous = contemporaneous,
                                temporal = temporal,
+                               scale = scale,
+                               nCores = 1,
+                               dayvar = dayvar,
+                               # beepvar = beepvar,
+                               verbose = FALSE,
+                               lags = 1)
+
+      # Both are specified
+    } else {
+
+      # Fit mlVAR
+      l_out_emp[[j]] <-  mlVAR(data = l_data[[j]],
+                               vars = vars,
+                               idvar = idvar,
+                               estimator = estimator,
+                               contemporaneous = contemporaneous,
+                               temporal = temporal,
+                               scale = scale,
                                nCores = 1,
                                dayvar = dayvar,
                                beepvar = beepvar,
                                verbose = FALSE,
                                lags = 1)
+
+
     } # end if: dayvar specified
 
+
+    # } else {
+    #
+    #   trueMeans_j <- trueMeans[trueMeans$id %in% l_data[[j]]$id, c(idvar, vars)]
+    #
+    #   # Fit mlVAR; note: true means spec currently only possible with mlVAR dev
+    #   l_out_emp[[j]] <-  mlVAR(data = l_data[[j]],
+    #                            vars = vars,
+    #                            idvar = idvar,
+    #                            estimator = estimator,
+    #                            contemporaneous = contemporaneous,
+    #                            temporal = temporal,
+    #                            scale = scale,
+    #                            trueMeans = trueMeans_j,
+    #                            nCores = 1,
+    #                            verbose = FALSE,
+    #                            lags = 1)
+    #
+    # } # end if: true means specified?
+
+    # } else {
+
+    # if(is.null(trueMeans)) {
+
+
+
+    # } else {
+    #
+    #   # Get true means for group J
+    #   trueMeans_j <- trueMeans[trueMeans$id %in% l_data[[j]]$id, c(idvar, vars)]
+    #
+    #   # Fit mlVAR; note: true means spec currently only possible with mlVAR dev
+    #   l_out_emp[[j]] <-  mlVAR(data = l_data[[j]],
+    #                            vars = vars,
+    #                            idvar = idvar,
+    #                            estimator = estimator,
+    #                            contemporaneous = contemporaneous,
+    #                            temporal = temporal,
+    #                            scale = scale,
+    #                            trueMeans = trueMeans_j,
+    #                            nCores = 1,
+    #                            dayvar = dayvar,
+    #                            beepvar = beepvar,
+    #                            verbose = FALSE,
+    #                            lags = 1)
+    #
+    # } # end if: true means specified?
+
+
+
   } # Loop: 2 groups
 
   # Final step
@@ -320,7 +442,7 @@ mlVAR_GC <- function(data, # data including both groups
 
     # b.1) VAR: fixed effects
     m_pval_phi_fix <- matrix(NA, p, p)
-    for(i in 1:p) for(j in 1:p) m_pval_phi_fix[i,j] <- mean(abs(a_phi_fixed[i,j,])>abs(diffs_true$diff_phi_fix[i,j,]))
+    for(i in 1:p) for(j in 1:p) m_pval_phi_fix[i,j] <- mean(abs(a_phi_fixed[i,j,]) > abs(diffs_true$diff_phi_fix[i,j,]))
 
     # b.2) VAR: RE sds
     m_pval_phi_RE_sd <- matrix(NA, p, p)
@@ -343,6 +465,9 @@ mlVAR_GC <- function(data, # data including both groups
 
     # ------ Create Output List -----
 
+    # Save the two mlVAR models estimated on the empirical data?
+    if(saveEmpModels==FALSE) l_out_emp <- NULL
+
     outlist <- list("Call" = Call,
                     "EmpDiffs" = list("Lagged_fixed" = diffs_true$diff_phi_fix,
                                       "Lagged_random" = diffs_true$diff_phi_RE_sd,
@@ -360,12 +485,11 @@ mlVAR_GC <- function(data, # data including both groups
                                       "Contemp_random" = a_gam_RE_sd,
                                       "Between" = a_between),
                     "Models" = l_out_ret,
+                    "EmpModels" = l_out_emp,
                     "Runtime_min" = runtime / 60)
 
   } # end if: permutation
 
-
-
   # ------ Compute p-values based on standard errors [parametric test] -----
   if(test == "parametric") {
 
@@ -374,8 +498,14 @@ mlVAR_GC <- function(data, # data including both groups
     bet_1se <- l_out_emp[[1]]$results$Beta$SE[, , 1]
     bet_2 <- l_out_emp[[2]]$results$Beta$mean[, , 1]
     bet_2se <- l_out_emp[[2]]$results$Beta$SE[, , 1]
+    # Compute degrees of freedom
+    if(partest == "Student") df = n_subj - 2
+    if(partest == "Welch") df = ((bet_1se^2 / n_subj_G1 + bet_2se^2 / n_subj_G2)^2) /
+      ((bet_1se^4 / (n_subj_G1^2 * (n_subj_G1 - 1))) + (bet_2se^4 / (n_subj_G2^2 * (n_subj_G2 - 1))))
+    # Compute t-statistic
     t_stat <- abs(bet_1-bet_2)/ sqrt(bet_1se^2 + bet_2se^2)
-    m_pval_phi_fix <- pt(t_stat, df=n_subj-2, lower.tail = FALSE) * 2 # times two to make 2-sided
+    # get pvalues
+    m_pval_phi_fix <- pt(t_stat, df = df, lower.tail = FALSE) * 2 # times two to make 2-sided
 
     # --- Contemporaneous effects ---
     # Average across nodewise reg to get estimates and SEs
@@ -383,8 +513,14 @@ mlVAR_GC <- function(data, # data including both groups
     bet_1se <- (l_out_emp[[1]]$results$Gamma_Theta$SE + t(l_out_emp[[1]]$results$Gamma_Theta$SE)) / 2
     bet_2 <-  (l_out_emp[[2]]$results$Gamma_Theta$mean + t(l_out_emp[[2]]$results$Gamma_Theta$mean)) / 2
     bet_2se <- (l_out_emp[[2]]$results$Gamma_Theta$SE + t(l_out_emp[[2]]$results$Gamma_Theta$SE)) / 2
+    # Compute degrees of freedom
+    if(partest == "Student") df = n_subj-2
+    if(partest == "Welch") df = ((bet_1se^2 / n_subj_G1 + bet_2se^2 / n_subj_G2)^2) /
+      ((bet_1se^4 / (n_subj_G1^2 * (n_subj_G1 - 1))) + (bet_2se^4 / (n_subj_G2^2 * (n_subj_G2 - 1))))
+    # Compute t-statistic
     t_stat <- abs(bet_1-bet_2)/ sqrt(bet_1se^2 + bet_2se^2)
-    m_pval_gam_fixed <- pt(t_stat, df=n_subj-2, lower.tail = FALSE) * 2 # times two to make 2-sided
+    # get pvalues
+    m_pval_gam_fixed <- pt(t_stat, df = df, lower.tail = FALSE) * 2 # times two to make 2-sided
     m_pval_gam_fixed[upper.tri(m_pval_gam_fixed)] <- NA
     diag(m_pval_gam_fixed) <- NA
 
@@ -394,14 +530,24 @@ mlVAR_GC <- function(data, # data including both groups
     bet_1se <- (l_out_emp[[1]]$results$Gamma_Omega_mu$SE + t(l_out_emp[[1]]$results$Gamma_Omega_mu$SE)) / 2
     bet_2 <- (l_out_emp[[2]]$results$Gamma_Omega_mu$mean + t(l_out_emp[[2]]$results$Gamma_Omega_mu$mean)) / 2
     bet_2se <- (l_out_emp[[2]]$results$Gamma_Omega_mu$SE + t(l_out_emp[[2]]$results$Gamma_Omega_mu$SE)) / 2
+    # Compute degrees of freedom
+    if(partest == "Student") df = n_subj-2
+    if(partest == "Welch") df = ((bet_1se^2 / n_subj_G1 + bet_2se^2 / n_subj_G2)^2) /
+      ((bet_1se^4 / (n_subj_G1^2 * (n_subj_G1 - 1))) + (bet_2se^4 / (n_subj_G2^2 * (n_subj_G2 - 1))))
+    # Compute t-statistic
     t_stat <- abs(bet_1-bet_2)/ sqrt(bet_1se^2 + bet_2se^2)
-    m_betw_sign <- pt(t_stat, df=n_subj-2, lower.tail = FALSE) * 2 # times two to make 2-sided
+    # get p-values
+    m_betw_sign <- pt(t_stat, df = df, lower.tail = FALSE) * 2 # times two to make 2-sided
     m_betw_sign[upper.tri(m_pval_gam_fixed)] <- NA
     diag(m_betw_sign) <- NA
 
 
     # ------ Create Output List -----
 
+    # Save the two mlVAR models estimated on the empirical data?
+    if(saveEmpModels==FALSE) l_out_emp <- NULL
+
+
     outlist <- list("Call" = Call,
                     "EmpDiffs" = list("Lagged_fixed" = diffs_true$diff_phi_fix,
                                       "Lagged_random" = diffs_true$diff_phi_RE_sd,
@@ -411,6 +557,7 @@ mlVAR_GC <- function(data, # data including both groups
                     "Pval" = list("Lagged_fixed" = m_pval_phi_fix,
                                   "Contemp_fixed" = m_pval_gam_fixed,
                                   "Between" = m_betw_sign),
+                    "EmpModels" = l_out_emp,
                     "Runtime_min" = runtime / 60)
 
   } # end if: test=parametric
